import { FC, PropsWithChildren, useCallback, useMemo } from "react";

import { Global, css } from '@emotion/react'
import { ThemeProvider } from '@mui/material/styles';
import createCache from '@emotion/cache';
import { CacheProvider } from '@emotion/react';
import useMediaQuery from '@mui/material/useMediaQuery';

import nuskinTheme from "../themes/genome-foundation";
import { generateGlobalColorCss, createNuskinTheme } from "../utils";

import { NuskinTheme } from "../themes/types";

type NuskinThemeProviderProps = {
  isSSR?: boolean;
  cacheKey?: string;
  theme?: NuskinTheme;
};

const getCacheKey = (key: string) =>  createCache({ key });

const NuskinThemeProvider: FC<PropsWithChildren<NuskinThemeProviderProps>> = (props) => {
    const {
        isSSR = false,
        children,
        cacheKey = 'nuskin-default',
        theme = nuskinTheme,
    } = props;

    const prefersDarkMode = useMediaQuery('(prefers-color-scheme: dark)');
    
    const appMuiTheme = useMemo(() => {
        const { palette } = theme || {};
        return createNuskinTheme(palette, prefersDarkMode);
    }, [prefersDarkMode, theme]);

    const cache = useMemo(() => getCacheKey(cacheKey), [cacheKey]);

    const renderThemeProvider = useCallback(() => (
        <ThemeProvider theme={appMuiTheme || {}}>
            <Global
                styles={css`
                  ${generateGlobalColorCss(theme)}
                `}
            />
            {children}
        </ThemeProvider>
    ), [appMuiTheme, theme, children]);

    if (isSSR) {
        return (
            <CacheProvider value={cache}>
                { renderThemeProvider() }
            </CacheProvider>
        )
    }
    return renderThemeProvider();
};

export default NuskinThemeProvider;
