import { createTheme } from "@mui/material";

import nuskinTheme from "../themes/genome-foundation/";
import { PaletteColorNames, Colors, ColorType, NuskinTheme } from "../themes/types";

const {
    palette: defaultPalette,
} = nuskinTheme;

export const generatePalette = (colors: Colors, prefersDarkMode = false) => {
    const colorKeys = Object.keys(colors) as PaletteColorNames[];
    const paletteColors = colorKeys.reduce((result, paletteName) => {
        if (prefersDarkMode) {
            const colorObject = colors[paletteName];
            const darkMode = colorObject?.darkMode;
            if (darkMode) {
                result[paletteName] = darkMode;
            }
        }
        return result;
    }, {} as Colors);
    return paletteColors;
};

export const createNuskinTheme = (palette: Colors = defaultPalette, prefersDarkMode?: boolean) => {
    const muiTheme = createTheme({ palette });
    return {
        ...muiTheme,
        palette: {
            ...muiTheme.palette,
            ...generatePalette(palette, prefersDarkMode),
        }
    };
};

const iterateNuskinThemeColors = (
    colorObj: ColorType | { [key: string]: string } = {},
) => {
    const allThemeColors: { [key: string]: string } = {};

    const  iterate = (colorObj: ColorType | { [key: string]: string } = {}, parentColor?: string) => {
        Object.keys(colorObj).forEach(key => {
            const colorKey = parentColor ? `${parentColor}-${key}` : key;
            const colorValue = colorObj[key];
            if (typeof colorValue === "object") {
                iterate(colorValue, colorKey);
            } else {
                allThemeColors[colorKey] = colorValue;
            }
        });
    }

    iterate(colorObj);
    return allThemeColors;
}

const generateColors = (name: string, value: string) => {
    return `
        .bg-${name} {
            background: ${value};
        }
        .text-${name} {
            color: ${value};
        }
        .border-${name} {
            border-color: ${value};
        }
    `;
};

export const generateGlobalColorCss = (theme: NuskinTheme) => {
    if (!theme) {
        return "";
    }
    
    let colorCssString = "";
    /*
     * Nuskin Theme consists of palette, which is sent to Mui
     * The Nuskin Theme also has colors which is used to generate the helper css
     */
    const palette = theme?.palette;
    
    const themeColors = iterateNuskinThemeColors(theme?.colors);

    const paletteColorNames = Object.keys(palette || {}) as PaletteColorNames[];
    
    const paletteColors = paletteColorNames?.filter((color: PaletteColorNames) => {
        const colorInPalette = palette?.[color];
        return colorInPalette?.main;
    });

    colorCssString = paletteColors?.reduce((result, colorName) => {
        const colorValue = palette?.[colorName]?.main;
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, "");

    colorCssString = Object.keys(themeColors)?.reduce((result, colorName: string) => {
        const colorValue = themeColors?.[colorName];
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, colorCssString);

    return colorCssString;
};
